	list p=16f84
;**************************************************************
;*
;* Pinbelegung
;*	----------------------------------	
;*	PORTA: 	0 Ziffer 1	-----------------------------------------------------+
;*		1 Ziffer 2	-------------------------------------+               I
;*		2 Ziffer 3	---------------------+               I               I
;*		3 Ziffer 4	-----+               I               I               I
;*		4 RC-5-Eingang	     I               I               I               I  
;*	PORTB:	0 Segment B	   AAAAA	   AAAAA	   AAAAA	   AAAAA
;*		1 Segment F	  F     B	  F     B	  F     B	  F     B
;*		2 Segment A	  F     B	  F     B	  F     B	  F     B
;*		3 Segment G	   GGGGG  	   GGGGG  	   GGGGG  	   GGGGG  
;*		4 Segment H	  E     C	  E     C	  E     C	  E     C
;*		5 Segment C	  E     C	  E     C	  E     C	  E     C
;*		6 Segment E	   DDDDD  HH	   DDDDD  HH	   DDDDD  HH	   DDDDD  HH
;*		7 Segment D    
;*
;**************************************************************
;
; sprut (zero) Bredendiek 06/2001  
; www.sprut.de
;
; 4-stellige RC-5-Anzeige
; Takt 4 MHz
;
; es wird probehalber "rc-5" angezeigt
;
;
; RC5-code hatt 5 Adressenbits (A0-A4) und 7 Commandobits (C0-C6)
; und ist wie folgt als 14-bit code aufgebaut:
;  1. startbit=1
;  2. invertiertes C6
;  3. Toggelbit: wechselt seinen wert von code zu code
;  4. A4
;  5. A3
;  6. A2
;  7. A1
;  8. A0
;  9. C5
; 10. C4
; 11. C3
; 12. C2
; 13. C1
; 14. C0
;
;**************************************************************
; Includedatei fr den 16F84 einbinden

	#include <P16f84a.INC>
; Configuration festlegen
;
; bis 4 MHz: Power on Timer, kein Watchdog, XT-Oscillator
	__CONFIG	_PWRTE_ON & _WDT_OFF & _XT_OSC
;
; ab 4 MHz: Power on Timer, kein Watchdog, HS-Oscillator
;	__CONFIG	_PWRTE_ON & _WDT_OFF & _HS_OSC

; Variablennamen vergeben

w_copy	Equ	0x20		; Backup fr Akkuregister
s_copy	Equ	0x21		; Backup fr Statusregister
Ziffer1	Equ	0x22		; Wert des LSD
Ziffer2	Equ	0x23		; Wert der zweitkleinsten Stelle
Ziffer3	Equ	0x24		; Wert der zweitgrten Stelle
Ziffer4	Equ	0x25		; Wert des MSD
Digit	Equ	0x26		; Ziffernzhler
ar	Equ	0x27
gr_e	Equ	0x28		; empfangene gruppe
cycl_1	Equ	0x29		; zhler
cycl_2	Equ	0x2A		; zhler
cycl_3	Equ	0x2B		; zhler fe empfangsbitschleife
data_1	Equ	0x2C		; datenpuffer low-teil
data_2	Equ	0x2D		; datenpuffer high-teil
co_e	Equ	0x2E		; empfangenes commando

;**************************************************************
; los gehts mit dem Programm

	org 	0
	goto	Init	

;**************************************************************
; die Interuptserviceroutine

	org 	4 	
intvec	bcf	INTCON, GIE	; disable Interupt 

	movwf	w_copy		; w retten
	swapf	STATUS, w 	; STATUS retten
	movwf	s_copy		;

	movlw	D'131'		; 256-125=131 ((1MHz : 32 ): 125 = 250 Hz)
	movwf	TMR0

; Intrupt servic routine
Int_serv 

	bsf	PORTA, 0	; Ziffer1 aus
	bsf	PORTA, 1	; Ziffer2 aus
	bsf	PORTA, 2	; Ziffer3 aus
	bsf	PORTA, 3	; Ziffer4 aus

	decf	Digit,f		; Ziffernzhler verringern
;Digit=4: anzeigen Ziffer 4
;Digit=3: anzeigen Ziffer 3
;Digit=2: anzeigen Ziffer 2
;Digit=1: anzeigen Ziffer 1
;Digit=0: andere Aktionen, keine Anzeige

	btfsc	STATUS, Z
	goto	Int_0		; Z-Flag=1 ergo Digit=0
	movfw	Digit
	movwf	ar
	decf	ar, f
	btfsc	STATUS, Z
	goto	Int_1		; Digit=1
	decf	ar, f
	btfsc	STATUS, Z
	goto	Int_2		; Digit=2
	decf	ar, f
	btfsc	STATUS, Z
	goto	Int_3		; Digit=3
	goto	Int_4		; Digit=4

Int_0
	movlw	5
	movwf	Digit
	goto	Int_end

Int_1
	movfw	Ziffer1		; Wert der 1. Ziffer
	call	Segmente
	movwf	PORTB		; Segmente einschalten
	bcf	PORTA, 0	; 1. Ziffer einschalten
	goto	Int_end

Int_2
	movfw	Ziffer2		; Wert der 2. Ziffer
	call	Segmente
	movwf	PORTB		; Segmente einschalten
	bcf	PORTA, 1	; 2. Ziffer einschalten
	goto	Int_end

Int_3
	movfw	Ziffer3		; Wert der 3. Ziffer
	call	Segmente
	movwf	PORTB		; Segmente einschalten
	bcf	PORTA, 2	; 3. Ziffer einschalten
	goto	Int_end

Int_4
	movfw	Ziffer4		; Wert der 4. Ziffer
	call	Segmente
	movwf	PORTB		; Segmente einschalten
	bcf	PORTA, 3	; 4. Ziffer einschalten
	goto	Int_end


Int_end	swapf	s_copy, w	; STATUS zurck
	movwf	STATUS 
	swapf	w_copy, f	; w zurck mit flags
	swapf	w_copy, w

	bcf	INTCON, T0IF	; Interupt-Flag lschen
	bsf	INTCON, GIE	; enable Interupt

	retfie

; 7-Segment-Tabelle
Segmente
	addwf	PCL, f
	retlw	B'00011000'	; 0
	retlw	B'11011110'	; 1
	retlw	B'00110010'	; 2
	retlw	B'01010010'	; 3
	retlw	B'11010100'	; 4
	retlw	B'01010001'	; 5
	retlw	B'00010001'	; 6
	retlw	B'11011010'	; 7
	retlw	B'00010000'	; 8
	retlw	B'01010000'	; 9
	retlw	B'10010000'	; A
	retlw	B'00010101'	; b
	retlw	B'00111001'	; C
	retlw	B'00010110'	; d
	retlw	B'00110001'	; E
	retlw	B'10110001'	; F
	retlw	B'11101111'	; .
	retlw	B'10110111'	; r
	retlw	B'00110111'	; c
	retlw	B'11110111'	; -
;**************************************************************

; Port A/B auf Ausgabe stellen

Init
	movlw	B'11111111'
	movwf	PORTA
	movwf	PORTB		; Anzeige dunkel
	bsf     STATUS, RP0	; auf Bank 1 umschalten
	movlw	B'11110000'	; PortA RA0-RA3 output
	movwf	TRISA
	movlw	B'00000000'	; PortB alle output
	movwf	TRISB
	bcf     STATUS, RP0	; auf Bank 0 zurckschalten

	clrf	gr_e
	clrf	co_e

; eine beliebige Zahl einstellen (hier: 'rc-5')
	movlw	5
	movwf	Ziffer1
	movlw	D'19'
	movwf	Ziffer2
	movlw	D'18'
	movwf	Ziffer3
	movlw	D'17'
	movwf	Ziffer4

	movlw	5
	movwf	Digit

; 250 Hz-Timer-Interupt einstellen
	bsf     STATUS, RP0	; auf Bank 1 umschalten
	movlw	B'10000100'	; internen Takt zhlen, Vorteiler zum Timer0, 32:1
	movwf	OPTION_REG
	movlw	D'131'		; 256-125=131 ((1MHz : 32 ): 125 = 250 Hz)
	bcf     STATUS, RP0	; auf Bank 0 zurckschalten
	movwf	TMR0
	bsf	INTCON, T0IE	; Timer0 interupt erlauben
	bsf	INTCON, GIE	; Interupt erlauben


;**********************************************************
;**********************************************************
;*
;* pollen der leitung
;*
;* RC-5 bus pollen
;* empfangene signale auf LED ausgeben
;*
;**********************************************************
;**********************************************************

pollen
	call	empfangen	;IR empfangen

	movfw	gr_e		; gruppenadresse 
	andlw	0x0F
	movwf	Ziffer3		; low Teil Gruppe

	swapf	gr_e,w		; gruppenadresse
	andlw	0x0F
	movwf	Ziffer4		; high Teil Gruppe

	movfw	co_e
	andlw	0x0F
	movwf	Ziffer1		; low Teil commando

	swapf	co_e,w		; commando
	andlw	0x0F
	movwf	Ziffer2		; high Teil commando

	call	pause		;2ms warten
	goto	pollen


;**********************************************************
;
; einen RC-5-code mit polling empfangen
;
; gruppencode: gr_e
; commando   : co_e
;
;**********************************************************
; vor dem aufruf wenigstens 1 ms low pegel abwarten !!
empfangen
	btfss	PORTA, 4
	goto	empfangen	; warten auf high

empfangenL
	btfsc	PORTA, 4
	goto	empfangenL	; warten auf low, 1/2-Bit vorbei

	bcf	INTCON,GIE	; interupts sperren

	call	viertelbit	; 1/4 nach bitmitte

	btfsc	PORTA, 4
	goto	Fehler		; war nur spike

	clrf	data_1		; datenpuffer lschen
	clrf	data_2
	movlw	D'13'		; 13 bits bei syncronisation
	movwf	cycl_3
;
; 3/4 nach bitanfang wird wert gelesen, invertiert
; und eingeschrieben
; dann wird auf die flanke gewartet, die das neue bit einleitet
; nach der flanke wird bis 3/4 bit gewartet und dann neu begonnen
;
	call	halbbit		; 1/4 des 2. Bits

nextbit
	rlf	data_1,f
	rlf	data_2,f
	bcf	data_1, 0	; angenommen leitung ist low (0) 
	btfsc	PORTA, 4	; leitung abfragen
	bsf	data_1, 0	; leitung ist high (1), in datenpuffer

; warten auf pegelnderung
	call	anders		; Bitmitte

	call	viertelbit	; 3/4 Bit
	call	halbbit		; 1/4 nach bitanfang
	decfsz	cycl_3,f	; alle 13 bit + startbit empfangen?
	goto	nextbit

	movfw	data_1
	movwf	co_e
	bcf	co_e, 6
	bcf	co_e, 7		; auf 6 bit begrenzen

	rlf	data_1,f
	rlf	data_2,f	
	rlf	data_1,f
	rlf	data_2,f	; gruppe nach data_2
	movfw	data_2
	movwf	gr_e
	bcf	gr_e, 7
	bcf	gr_e, 6
	bcf	gr_e, 5		; auf 5 bit begrenzen
	
; 2. stoppbit zum 7 commandobit machen
	btfss	data_2, 6
	bsf	co_e, 6		;2. Stoppbit invers  -> 7.commandobit

	movfw	co_e
Fehler
	bsf	INTCON,GIE	; interrupts wieder erlauben

	return

;**************************************************************
;
; warten darauf, dass die leitung RA0 dem inversen wert in 'data_1, 0' entspricht
;
; zeitbegrenzt auf ca. ein halbes bit
;
;**************************************************************
anders
;	movlw	D'22'		; 22 x (10 MHz)
	movlw	D'9'		; 9 x  ( 4 MHz)
	movwf	cycl_1
a_1
	movlw	D'32'		; 100 takte
	movwf	cycl_2
a_2
	decfsz	cycl_2,f	
	goto	a_2

	btfsc	PORTA, 4	; portpegel abfragen
	goto	a_high
a_low				; portpegel ist 0
	btfsc	data_1, 0	; ist sollwert=0 ?
	return			; high-low flanke
	goto	a_weiter
a_high				; portpegel ist 1
	btfss	data_1, 0	; ist sollwert=1 ?
	return			; low-high flanke
a_weiter
	decfsz	cycl_1,f	
	goto	a_1	
	return			; keine flanke gefunden

;**************************************************************
;
; zeitverzgerung um ein viertelbit = 444,5 us ca. 1100 takte (10 MHz)
;						445 takte  (04 MHz)
;
; zeitverzgerung um ein halbbit    = 889 us   ca. 2200 takte (10 MHz)
;						889 takte  (04 MHz)
;
;**************************************************************
pause				;2ms fr 45
	movlw	D'45'
	goto	l_0
pause_l				;ca 12ms fr 255
	movlw	D'255'
	goto	l_0
viertelbit
;	movlw	D'11'		; 11 x (10 MHz)
	movlw	D'9'		; 9 x  ( 4 MHz)
	goto	l_0
halbbit
;	movlw	D'22'		; 22 x (10 MHz)
	movlw	D'18'		; 18 x ( 4 MHz)
l_0
	movwf	cycl_1
l_1

;	movlw	D'32'		; 100 takte (10 MHz)
	movlw	D'15'		; 49 takte  ( 4 MHz)
	movwf	cycl_2
l_2
	decfsz	cycl_2,f	
	goto	l_2

	decfsz	cycl_1,f	
	goto	l_1	
	return

	end



